<?php

# Copyright (C) 2009 Riku Kalinen
# License conditions in the file lisenssi.html
# Lisenssiehdot tiedostossa lisenssi.html

# sanity check functions for database additions, modifications and
# deletions.
# general format is ret = table_what_ok(&$msg,$id, ...)
# msg is error/warning message
# id is the main id of the record in question, new record => id = ''
# return codes:
#  0 (all ok, msg undefined)
#  1 (warnings in msg)
#  2 (fatal, reason in msg)

# laite tarkastusfunktiot
# HUOM: ryhma_id vlitetn viittauksena koska sit voidaan joutua muuttamaan

function laite_ok(&$msg,$id,$alue_id,$paivystys_id,$jarjesto_id,$hlo_id,$laite,
		  $tunniste,$verkko_id,&$ryhma_id,$laite_tyyppi_id) {
  global $table;
  
  $msg = '';
  $ret = 0;
  
  if ($id != "") {
    $id_sql = "AND id != $id";
    $modify = true;
    $tail = " Ei voi muuttaa.";
  } else {
    $modify = false;
    $tail = " Ei voi list.";
  }

  $ret_p = permission_ids($msg,$alue_id,$paivystys_id);
  if ($ret_p == 2) {
    $msg .= $tail;
    return 2;
  } else {
    $ret = $ret_p;
  }
  
  # jos muutos, tarkastetaan ett on oikeus mys oleviin tietoihin
  if ($modify) {
    $w_alue_id = tab_col_id($table,"alue_id",$id);
    $w_paivystys_id = tab_col_id($table,"paivystys_id",$id);
    $ret_p = permission_ids($msg,$w_alue_id,$w_paivystys_id);
    if ($ret_p == 2) {
      $msg .= $tail;
      return 2;
    } else {
      $ret = $ret_p;
    }
  } # modify
  
  # tarpeelliset kentt
  if (($jarjesto_id == 1) && ($hlo_id == 0)) {
    $msg = 'Laitteella on oltava omistaja, joko henkil tai jrjest tai
 molemmat. ';
    $msg .= $tail;
    return 2;
  }

  if ($laite == '') {
    $msg = 'Laitteen mallimerkint on pakollinen tieto.';
    $msg .= $tail;
    return 2;
  }

  # jarjesto, hlo, verkko, ryhma ja laite_tyyppi oltava olemassa
  # tai soveltuvin osin 0
  if (tab_col_id("jarjesto","",$jarjesto_id) == '') {
    $msg = "Jrjest $jarjesto_id ei lydy.";
    $msg .= $tail;
    return 2;
  }
  if (tab_col_id_nobody("hlo","nimi",$hlo_id) == '') {
    $msg = "Henkil $hlo_id ei lydy.";
    $msg .= $tail;
    return 2;
  }
  if (tab_col_id("verkko","",$verkko_id) == '') {
    $msg = "Verkkoa $verkko_id ei lydy.";
    $msg .= $tail;
    return 2;
  }
  if (tab_col_id("ryhma","kutsu",$ryhma_id) == '') {
    $msg = "Ryhm $ryhma_id ei lydy.";
    $msg .= $tail;
    return 2;
  }
  if (tab_col_id("laite_tyyppi","",$laite_tyyppi_id) == '') {
    $msg = "Laitetyyppi $laite_tyyppi_id ei lydy.";
    $msg .= $tail;
    return 2;
  }
  
  # duplikaatit
  $dup =
    ota_yksi("SELECT id
              FROM $table
              WHERE jarjesto_id = $jarjesto_id AND hlo_id = $hlo_id
                    AND laite='$laite' AND tunniste = '$tunniste'
                    AND verkko_id = $verkko_id
                    AND laite_tyyppi_id = $laite_tyyppi_id
              $id_sql");
  if ($dup != '') {
    $msg = 'Saman omistajan laitteiden on oltava yksilllisi, eli vhintn '
      . 'tunnisteiden on erottava toisistaan.';
    if ($modify) {
      $msg .= " Tllainen laite on jo olemassa (id=$dup).";
    }
    $msg .= $tail;
    return 2;
  } 

  # jos pivystys asetettu, tytyy tsmt alueeseen
  if ($paivystys_id > 2) {
    $paivystyksen_alue = tab_col_id("paivystys","alue_id",$paivystys_id);
    if (($paivystyksen_alue != 2) && ($paivystyksen_alue != $alue_id)) {
      $msg = "Alue($alue_id) ja operaatio($paivystys_id) eivt sovi yhteen.";
      $msg .= $tail;
      return 2;
    }
  }
  
  # jos ryhma ja paivystys on asetettu, niiden tytyy tsmt.
  if (($ryhma_id != '1') && ($paivystys_id != '1')) {
    if (tab_col_id("ryhma","paivystys_id",$ryhma_id) != $paivystys_id) {
      $msg = "Annetut ryhm($ryhma_id) ja operaatio($paivystys_id) eivt
 sovi yhteen. Ryhm palautettu oletustilaan.";
      $ryhma_id = 1;
      $ret = 1;
    }
  }

  return $ret;
}

function laite_add_ok(&$msg,$alue_id,$paivystys_id,$jarjesto_id,$hlo_id,$laite,
		      $tunniste,$verkko_id,&$ryhma_id,$laite_tyyppi_id) {
  return laite_ok($msg,'',$alue_id,$paivystys_id,$jarjesto_id,$hlo_id,$laite,
		  $tunniste,$verkko_id,$ryhma_id,$laite_tyyppi_id);
}

function laite_save_ok(&$msg,$id,$alue_id,$paivystys_id,$jarjesto_id,$hlo_id,
		       $laite,$tunniste,$verkko_id,&$ryhma_id,
		       $laite_tyyppi_id) {
  return laite_ok($msg,$id,$alue_id,$paivystys_id,$jarjesto_id,$hlo_id,$laite,
		  $tunniste,$verkko_id,$ryhma_id,$laite_tyyppi_id);
}

function laite_remove_ok(&$msg, $id, $log = true) {
  global $table;
  
  $ret = 0;
  
  $ret_p = permission_tab($msg, $table, $id, $log);
  if ($ret_p == 2) {
    return 2;
  } else {
    $ret = $ret_p;
  }
  
  return $ret;
}

?>

<!-- RCS $Id: laite-ok.php,v 1.6 2009/03/24 09:11:02 riku Exp $ -->
